/*
 * File    : led.c
 * Purpose : define functions to control LEDs
 */

#include <avr/io.h>
#include <avr/signal.h>
#include <avr/interrupt.h>

#include "led.h"
#include "types.h"

/* define ports and pins for LEDs */
#define LED_DDR    DDRA
#define LED_PORT   PORTA
#define GREEN_PORT PA7
#define RED_PORT   PA6
#define GREEN_PIN  6
#define RED_PIN    7

/* delay counter for each led */
static int  led_stat[2] = { 0, 0 };

#define LED_DELAY  500

/*
 * init LEDs
 */
void
led_init ()
{
    sbi( LED_DDR, GREEN_PORT );
    sbi( LED_DDR, RED_PORT );

    led_off( LED_GREEN );
    led_off( LED_RED );

    /* timer off */
    TCCR0  = 0;
    /* set counter (100 = 256 - 156) */
    TCNT0  = 156;
    /* enable interrupt */
    TIMSK |= (1 << TOIE0);
}

void
led_on ( char n )
{
    switch ( n )
    {
    case LED_GREEN: cbi( LED_PORT, GREEN_PIN ); break;
    case LED_RED  : cbi( LED_PORT, RED_PIN );   break;
    }
}

void
led_off ( char n )
{
    switch ( n )
    {
    case LED_GREEN: sbi( LED_PORT, GREEN_PIN ); break;
    case LED_RED  : sbi( LED_PORT, RED_PIN );   break;
    }
}

void
led_off_d ( char n )
{
    switch ( n )
    {
    case LED_GREEN: led_stat[LED_GREEN] = LED_DELAY; break;
    case LED_RED  : led_stat[LED_RED]   = LED_DELAY; break;
    default       : return;
    }

    /* start timer, clock/8 prescaling */
    TCCR0 = (1 << CS01);
}

/*
 * interrupt routine for delayed led off
 */

SIGNAL( SIG_OVERFLOW0 )
{
    uchar  i;
    bool   active = false;
    
    for ( i = 0; i < 2; i++ )
    {
        if ( led_stat[i] == 0 )
            led_off( i );
        else
        {
            led_stat[i]--;
            active = true;
        }
    }

    if ( active )
        TCNT0 = 156;  /* re-initialise timer */
    else
        TCCR0 = 0;    /* stop timer */
}

